require "yaml"

class Theme < Hash
  @uti = nil
  
  def initialize(doc)
    @uti = doc.keys[0]
    replace(doc[@uti])
  end
  
  def value_for_id(id, id_root=nil)
    return if id.length < 0
    root = self
    if id.length > 1 and id[0..1] == ".."
      # xxx
      return '[xxx todo resolve ..+]'
    elsif id[0] == '.'[0]
      root = id_root
      path = id[1..id.length].split('.')
    else
      path = id.split('.')
    end
    path.each { |n|
      if root.key? n
        root = root[n]
      else
        raise NameError.new("#{n} (#{id})")
      end
    }
    root
  end
  
  def resolve_references!
    replace(resolve_h(self))
  end
  
  def resolve_h(h)
    h.each_pair { |k, v|
      is_ref = (v.is_a? String and v.length > 0 and v[0] == '@'[0])
      is_parent_key = (k == "_")
      if is_parent_key and not is_ref
        raise NameError.new("\"_\"-key (parent) without \"@\"-value: #{k}: #{v.to_s}")
      end
      if is_ref
        nv = value_for_id(v[1..v.length], h)
        if is_parent_key
          h.delete(k)
          h.merge!(nv)
        else
          h[k] = nv
        end
        # xxx
      elsif v.is_a? Hash
        resolve_h v
      elsif v.is_a? Array
        resolve_l v
      end
    }
  end
  
  def resolve_l(l)
    puts '>> xxx todo implement resolve_l'
    # xxx todo
  end
  
  def dump_v(k, v)
    if v.is_a? Hash
      dump_h k, v
    elsif v.is_a? Array
      dump_l k, v
    #elsif v.is_a? Integer
    #  puts k + ': 0x%08x' % v
    else
      puts k + ': ' + v.to_s
    end
  end

  def dump_h(kbase, o)
    o.each_pair { |k, v|
      if kbase.length > 0
        dump_v kbase + '.' + k.to_s, v
      else
        dump_v k.to_s, v
      end
    }
  end

  def dump_l(kbase, o)
    o.each_with_index { |v, i|
      dump_v "#{kbase}[#{i.to_s}]", v
    }
  end
  
  def dump(include_uti=false)
    if include_uti
      dump_h @uti, self
    else
      dump_h '', self
    end
  end
end

File.open('../specification/theme.yaml') do |f|
  YAML.load_documents(f) do |doc|
    theme = Theme.new(doc)
    theme.resolve_references!
    theme.dump false
  end
end
